//!
//!
use crate::repositories::{
  bank::models::Bank as _Bank, member::models::Member as _Member,
  member::schema::MemberState as _MemberState, member_class::models::MemberClass as _MemberClass,
  member_level::models::MemberLevel as _MemberLevel, site::models::Site as _Site,
};
use diesel::deserialize::QueryableByName;

///
#[derive(Eq, Hash, PartialEq, Debug, Clone)]
pub struct MemberModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub site: _Site,
  ///
  pub member_class: _MemberClass,
  ///
  pub member_level: _MemberLevel,
  ///
  pub username: String,
  ///
  pub password: String,
  ///
  pub nickname: String,
  ///
  pub mobile_phone_number: Option<String>,
  ///
  pub state: _MemberState,
  ///
  pub state_changed_at: Option<i64>,
  ///
  pub bank_account: Option<MemberBankAccountModel>,
  ///
  pub referrer_member: Option<_Member>,
  ///
  pub referred_count: i64,
  ///
  pub last_signined_ip: Option<String>,
  ///
  pub last_signined_at: Option<i64>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
  ///
  pub deleted_at: Option<i64>,
}

impl QueryableByName<diesel::pg::Pg> for MemberModel {
  fn build<R: diesel::row::NamedRow<diesel::pg::Pg>>(row: &R) -> diesel::deserialize::Result<Self> {
    let referrer_member = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>(
        "m_referrer_member_id",
      )? {
      Some(_) => Some(_Member {
        id: row.get("_m_id")?,
        site_id: row.get("_m_site_id")?,
        member_class_id: row.get("_m_member_class_id")?,
        member_level_id: row.get("_m_member_level_id")?,
        username: row.get("_m_username")?,
        password: row.get("_m_password")?,
        nickname: row.get("_m_nickname")?,
        mobile_phone_number: row.get("_m_mobile_phone_number")?,
        state: row.get("_m_state")?,
        state_changed_at: row.get("_m_state_changed_at")?,
        referrer_member_id: row.get("_m_referrer_member_id")?,
        referred_count: row.get("_m_referred_count")?,
        last_signined_ip: row.get("_m_last_signined_ip")?,
        last_signined_at: row.get("_m_last_signined_at")?,
        created_at: row.get("_m_created_at")?,
        updated_at: row.get("_m_updated_at")?,
        deleted_at: row.get("_m_deleted_at")?,
      }),
      None => None,
    };

    let bank = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("_b_id")?
    {
      Some(_) => Some(_Bank {
        id: row.get("_b_id")?,
        name: row.get("_b_name")?,
        sort_order: row.get("_b_sort_order")?,
        show: row.get("_b_show")?,
        can_use: row.get("_b_can_use")?,
        memo: row.get("_b_memo")?,
        created_at: row.get("_b_created_at")?,
        updated_at: row.get("_b_updated_at")?,
      }),
      None => None,
    };

    let member_bank_account = match bank {
      Some(bank) => Some(MemberBankAccountModel {
        id: row.get("mba_id")?,
        member_id: row.get("mba_member_id")?,
        bank,
        name: row.get("mba_name")?,
        account_number: row.get("mba_account_number")?,
        memo: row.get("mba_memo")?,
        created_at: row.get("mba_created_at")?,
        updated_at: row.get("mba_updated_at")?,
      }),
      None => None,
    };

    Ok(MemberModel {
      id: row.get("m_id")?,
      site: _Site {
        id: row.get("s_id")?,
        url: row.get("s_url")?,
        name: row.get("s_name")?,
        path: row.get("s_path")?,
        show: row.get("s_show")?,
        can_use: row.get("s_can_use")?,
        memo: row.get("s_memo")?,
        expires_at: row.get("s_expires_at")?,
        created_at: row.get("s_created_at")?,
        updated_at: row.get("s_updated_at")?,
      },
      member_class: _MemberClass {
        id: row.get("mc_id")?,
        parent_id: row.get("mc_parent_id")?,
        name: row.get("mc_name")?,
        show: row.get("mc_show")?,
        created_at: row.get("mc_created_at")?,
        updated_at: row.get("mc_updated_at")?,
      },
      member_level: _MemberLevel {
        id: row.get("ml_id")?,
        name: row.get("ml_name")?,
        show: row.get("ml_show")?,
        sort_order: row.get("ml_sort_order")?,
        created_at: row.get("ml_created_at")?,
        updated_at: row.get("ml_updated_at")?,
      },
      username: row.get("m_username")?,
      password: row.get("m_password")?,
      nickname: row.get("m_nickname")?,
      mobile_phone_number: row.get("m_mobile_phone_number")?,
      state: row.get("m_state")?,
      state_changed_at: row.get("m_state_changed_at")?,
      bank_account: member_bank_account,
      referrer_member,
      referred_count: row.get("m_referred_count")?,
      last_signined_ip: row.get("m_last_signined_ip")?,
      last_signined_at: row.get("m_last_signined_at")?,
      created_at: row.get("m_created_at")?,
      updated_at: row.get("m_updated_at")?,
      deleted_at: row.get("m_deleted_at")?,
    })
  }
}

#[derive(Eq, Hash, PartialEq, Debug, Clone)]
pub struct MemberBankAccountModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub member_id: uuid::Uuid,
  ///
  pub bank: _Bank,
  ///
  pub name: String,
  ///
  pub account_number: String,
  ///
  pub memo: Option<String>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
}
